import conf
import requests
import logging

logger = logging.getLogger(__name__)


def get_legal_tag_list(record):
    if "legal" in record and "legaltags" in record["legal"]:
        return record["legal"]["legaltags"]
    return []


def get_legal_tag_info(legal_tags, data_partition_id, access_token):
    """Get legal tag details"""
    if conf.LEGAL_BATCH_API is None or not legal_tags:
        logger.debug("unexpected legal tag or API")
        return []

    headers = {
        "Authorization": "Bearer " + access_token,
        "data-partition-id": data_partition_id,
        "Content-type": "application/json",
        "accept": "application/json",
    }

    legal_tag_response = requests.post(
        url=conf.LEGAL_BATCH_API, headers=headers, json={"names": legal_tags}
    )

    if legal_tag_response.ok:
        logger.debug(f"OK {legal_tag_response.text}")
        if "legalTags" in legal_tag_response.json():
            return legal_tag_response.json()["legalTags"]
        return []
    else:
        logger.error(
            f"ERROR legaltags {legal_tag_response.text} {legal_tag_response.status_code}"
        )
        return None
