import logging
from fastapi import Header
from fastapi import HTTPException
from starlette_context import request_cycle_context
from uuid_extensions import uuid7str
from conf import GENERATE_CORRELATION_ID, LOG_LEVEL, ALLOW_CORRELATION_ID_PATTERN
import re

from starlette.status import (
    HTTP_400_BAD_REQUEST,
)
# import coloredlogs

# class CorrelationIDLoggerAdapter(logging.LoggerAdapter):
#    def __init__(self, logger, prefix):
#        super(CorrelationIDLoggerAdapter, self).__init__(logger, {})
#        self.prefix = prefix
#
#    def process(self, msg, kwargs):
#        return '[%s] %s' % (self.prefix, msg), kwargs


class DefaultExtrasAdapter(logging.LoggerAdapter):
    def __init__(self, logger, extra, level=LOG_LEVEL):
        super(DefaultExtrasAdapter, self).__init__(logger, extra)
        # field_styles = coloredlogs.DEFAULT_FIELD_STYLES
        # field_styles['correlation_id'] = {'color': 'magenta'}
        # coloredlogs.install(logger=logger, level=level, fmt='%(asctime)s %(hostname)s logger=%(name)s loglevel=%(levelname)-6s correlation-id=%(correlation_id)s %(funcName)s() L%(lineno)-4d %(message)s   call_trace=%(pathname)s L%(lineno)-4d')
        # logger=logger, level=level, fmt='%(asctime)s %(hostname)s logger=%(name)s loglevel=%(levelname)-6s correlation-id=%(correlation_id)s %(funcName)s() L%(lineno)-4d %(message)s   call_trace=%(pathname)s L%(lineno)-4d'
        # extra = {'correlation_id': }

    def process(self, msg, kwargs):
        # Speed gain if no extras are present
        if "extra" in kwargs:
            copy = dict(self.extra).copy()
            copy.update(kwargs["extra"])
            kwargs["extra"] = copy
        else:
            kwargs["extra"] = self.extra
        return msg, kwargs


async def policy_context_dependency(
    correlation_id: str = Header(None), user_agent: str = Header(None)
):
    # When used a Depends(), this function get the `Correlation-ID` header,
    # which will be documented as a required header by FastAPI.
    # use `correlation_id: str = Header(None)` for an optional header.
    # (correlation_id = Header(...) for required header

    if not correlation_id and GENERATE_CORRELATION_ID:
        correlation_id = uuid7str()
    else:
        if not re.match(pattern=ALLOW_CORRELATION_ID_PATTERN, string=correlation_id):
            raise HTTPException(
                status_code=HTTP_400_BAD_REQUEST, detail="Unsupported correlation-id"
            )

    data = {"correlation_id": correlation_id, "user_agent": user_agent}
    with request_cycle_context(data):
        # yield allows it to pass along to the rest of the request
        yield


old_factory = logging.getLogRecordFactory()


def set_correlation_id(correlation_id=None):
    def record_factory(*args, **kwargs):
        record = old_factory(*args, **kwargs)
        record.correlation_id = correlation_id
        return record

    return record_factory
