import io
import logging


class StringLogger:
    def __init__(self, logger_name: str):
        self.logger_name = logger_name
        self.log_stringio = io.StringIO()

        # create stream log handler with string i/o buffer
        self.log_handler = logging.StreamHandler(self.log_stringio)
        self.logger = self._string_io_logger()

    def __repr__(self):
        return "StringLogger()"

    def __str__(self):
        return self.log_stringio.getvalue()

    def _string_io_logger(self):
        # create logger
        logger = logging.getLogger(self.logger_name)
        formatter = logging.Formatter(
            f"%(asctime)s %(levelname)s \t[%(filename)s:%(lineno)s - %(funcName)s()] {self.logger_name}: %(message)s"
        )
        logger.setLevel(logging.INFO)

        # add normal stream handler to display logs on screen
        # for testing this code
        # io_log_handler = logging.StreamHandler()
        # io_log_handler.setFormatter(formatter)
        # logger.addHandler(io_log_handler)

        # create stream handler and initialise it with string io buffer
        string_io_log_handler = logging.StreamHandler(self.log_stringio)
        string_io_log_handler.setFormatter(formatter)

        # add stream handler to logger
        logger.addHandler(string_io_log_handler)
        return logger


if __name__ == "__main__":
    stringlog = StringLogger("mys3logger")
    stringlog.logger.info("this log goes to string i/o buffer....")

    print(f"Log message:\n{stringlog}")
