#!/usr/bin/env bash

TMP=/tmp
TMPDIR=/tmp/policy-init
DP_TARFILE=${TMP}/bundle-${DATA_PARTITION}.tar.gz
TARFILE=${TMP}/bundle.tar.gz
TEMPLATE_DIR=tests/templates
INSTANCE_DIR=tests/instance_data
TOKEN=$(gcloud auth print-identity-token)
AUTH_REGO=dataauthz
export TOKEN
mkdir -p $TMPDIR
SCRIPTDIR=$PWD

if [[ -z "${POLICY_BUCKET}" ]]; then
    echo "POLICY_BUCKET env not defined"
    exit 1
else
    echo "POLICY_BUCKET: gs://${POLICY_BUCKET}"
fi
if [[ -z "${DATA_PARTITION}" ]]; then
    echo "DATA_PARTITION env not defined"
    exit 1
else
    echo "DATA_PARTITION: ${DATA_PARTITION}"
fi

/bin/rm -f ${DP_TARFILE}
/bin/rm -f ${TARFILE}
/bin/rm -f ${TMPDIR}/*.rego ${TMPDIR}/.manifest

# rewrite templates
echo "Creating .manifest and .rego..."
#envsubst < ${TEMPLATE_DIR}/manifest.template > ${TMPDIR}/.manifest
envsubst < ${TEMPLATE_DIR}/manifest-with-instance.template > ${TMPDIR}/.manifest
envsubst < ${TEMPLATE_DIR}/${AUTH_REGO}.rego.template > ${TMPDIR}/${AUTH_REGO}.rego

cat ${TMPDIR}/${AUTH_REGO}.rego ${TMPDIR}/.manifest

echo "Achive ${DATA_PARTITION} bundle of policies..."
echo -------
echo `ls ${TMPDIR}`
echo -------
tar -cvzf ${DP_TARFILE} --directory=${TMPDIR}  --verbose . 
/bin/rm -f ${TMPDIR}/*.rego ${TMPDIR}/.manifest

opa inspect ${DP_TARFILE}
echo "Push data partition bundle to bucket..."
gsutil cp ${DP_TARFILE} gs://"${POLICY_BUCKET}"/
echo "Achive ${DATA_PARTITION} bundle of policies and push to bucket ${POLICY_BUCKET} - DONE!"
#rm -f ${DP_TARFILE}

echo $(ls ${TMPDIR})
echo "Gathering instance policies.."
cp ${INSTANCE_DIR}/*.rego ${TMPDIR}
cp ${INSTANCE_DIR}/manifest ${TMPDIR}/.manifest
echo $(ls ${TMPDIR})
echo "Achive instance bundle of policies..."
tar -cvzf ${TARFILE} --directory=${TMPDIR}  --verbose . 
opa inspect ${TARFILE}
echo "Push instance bundle to bucket..."
gsutil cp ${TARFILE} gs://"${POLICY_BUCKET}"/
echo "Achive instance bundle of policies and push to bucket ${POLICY_BUCKET} - DONE!"
#rm -f ${TARFILE}
