# Cache

## OPA Caching from Policy

Caching of [OPA](https://www.openpolicyagent.org/) responses was added in M16 to handle cases where OPA was undersized in terms of memory requests and number of replicas (pods). Caching should not be needed if OPA is sized large enough for your load, however you'll likely get better performance with caching enabled.

When search makes calls to Policy Service Translate API, Policy Service can make 1-2 calls to OPA per search request. There are also other APIs in policy that make calls to OPA and there are services that talk to OPA directly like search.

See [sizing](sizing.md) for more details on OPA Sizing recommendations.

### How to disable Policy Service Cache

To disable caching set environment variable `DISABLE_OPA_CACHE` to `True`.
Caching will also be disabled if the `OPA_CACHE_MAXSIZE` is 0

## Adjusting Policy Service Cache Settings
Caching of OPA responses can be controlled by the following environments variables. 
Shown with defaults:
```
OPA_CACHE_MAXSIZE=128
OPA_CACHE_TTL=120
OPA_LIST_CACHE_MAXSIZE=1
OPA_LIST_CACHE_TTL=60
OPA_FETCH_CACHE_MAXSIZE (by default inherits value of OPA_CACHE_MAXSIZE)
OPA_FETCH_CACHE_TTL=60
OPA_COMPILE_CACHE_TTL (by default inherits value of OPA_CACHE_TTL)
OPA_DATAAPI_CACHE_MAXSIZE (by default inherits value of OPA_CACHE_MAXSIZE)
OPA_DATAAPI_CACHE_TTL (by default inherits value of OPA_CACHE_TTL)
```

!!! info

    Prior to M20, to tweak cache settings required container image build with different settings in conf.py
    Details on how your policy service is configured, including cache settings, can be retrieved via the [/config API](api.md#get-apipolicyv1config)

### Policy List
Cache list of policies available, only cache latest result
```
OPA_LIST_CACHE_MAXSIZE=1
OPA_LIST_CACHE_TTL=60
```

### Policies Cache
Cache policies themselves from OPA
```
OPA_FETCH_CACHE_MAXSIZE=OPA_CACHE_MAXSIZE
OPA_FETCH_CACHE_TTL=60
```

### Compile and Data API Cache
Cache compile and dataapi results from OPA (used by translate/search)
```
OPA_COMPILE_CACHE_MAXSIZE=OPA_CACHE_MAXSIZE
OPA_COMPILE_CACHE_TTL=OPA_CACHE_TTL
OPA_DATAAPI_CACHE_MAXSIZE=OPA_CACHE_MAXSIZE
OPA_DATAAPI_CACHE_TTL=OPA_CACHE_TTL
```

### Cache Statics
Enabling any of the below will add CacheInfo (hits, misses, maxsize, and currsize) to config API output.
Note that cache_info will inflict a - probably minor - performance penalty, so it has to be explicitly enabled to policy service.
```
OPA_DATA_CACHE_INFO
OPA_COMPILE_CACHE_INFO 
OPA_DOCUMENT_CACHE_INFO 
OPA_FETCH_CACHE_INFO
OPA_LIST_CACHE_INFO
```

## Caching within Policies

To use caching with rego policies in [http.send](https://www.openpolicyagent.org/docs/latest/policy-reference/#http) you can use `cache` or `force_cache`.

If you use `force_cache`, please note this overrides cache directives defined by the OPA service. So you'll want to set `force_cache_duration_seconds`.

!!! warning "cache_ignored_headers"

    If a cache entry exists with a subset/superset of headers that are considered in this request, it will lead to a cache miss. Please consider using `cache_ignored_headers` when using caching. Please note, `cache_ignored_headers` was added in v0.66 release of OPA.

## OPA Cache Settings

!!! warning "Set OPA default cache settings"

    If you are using cache settings in policies it is strongly recommended to consider using these OPA config settings(not in Rego).
- `caching.inter_query_builtin_cache.max_size_bytes` to set a Inter-query cache size limit in bytes. OPA will drop old items from the cache if this limit is exceeded. By default, no limit is set (i.e. unlimited) and this could cause Out of Memory `OOMKilled` and `MemoryPressure` issues.
- `caching.inter_query_builtin_cache.forced_eviction_threshold_percentage` - Threshold limit configured as percentage of caching.inter_query_builtin_cache.max_size_bytes, when exceeded OPA will start dropping old items permaturely. By default, set to 100.
- `caching.inter_query_builtin_cache.stale_entry_eviction_period_seconds` - Stale entry eviction period in seconds. OPA will drop expired items from the cache every stale_entry_eviction_period_seconds. By default, set to 0 indicating stale entry eviction is disabled.